@@ -19,6 +19,8 @@ import logging
 import numpy as np
 import pandas as pd
 
+
+logger = logging.getLogger(__name__)
 #
 #
 #
@@ -31,7 +33,8 @@ _BINT = {'c': 'c', 'd': 'i', 'hd': 'h', 'x': 'i', 'hx': 'h',
          'f': 'f', 'lf': 'd', 'e': 'f', 'le': 'd', 't': 'i',
          'lt': 'f', }
 _BINL = {'c': 1, 'd': 4, 'hd': 2, 'x': 4, 'hx': 2,
-         'f': 4, 'lf': 8, 'e': 4, 'le': 8, 't': 4, 'lt': 8, }
+         'f': 4, 'lf': 8, 'e': 4, 'le': 8, 't': 4, 'lt': 8,
+         's': 1}
 _BINP = {'c': bytes, 'd': int, 'hd': int, 'x': int, 'hx': int,
          'f': float, 'lf': float, 'e': float, 'le': float, 't': int,
          'lt': float, }
@@ -96,8 +99,8 @@ def _parse_form(forms):
     rep_mark = r'\[[0-9]+\]'
     for f in forms:
         if re.search(rep_mark, f):
-            rep_count = int(re.sub(r'.*\[([0-9]+)\].*', r'\1', f))
-            rep_form = re.sub(r'\[([0-9]+)\]', '', f)
+            rep_count = int(re.sub(r'.*\[([0-9]+)].*', r'\1', f))
+            rep_form = re.sub(r'\[([0-9]+)]', '', f)
             for i in range(rep_count):
                 forms2.append(rep_form)
         else:
@@ -114,12 +117,12 @@ def _parse_form(forms):
     specs = []
     nbyte = []
     for f in forms:
-        logging.debug('parsing: "{}"'.format(f))
+        logger.debug('parsing: "{}"'.format(f))
         #     '
         # Name des Datenelementes (optional).
         if '%' in f:
             x, f = f.split('%')
-            logging.debug('... name  : "{}"'.format(x))
+            logger.debug('... name  : "{}"'.format(x))
         else:
             x = ''
         nams.append(x)
@@ -129,7 +132,7 @@ def _parse_form(forms):
         if ')' in f:
             x = re.sub(r'\(\*(.*)\).*', r'\1', f)
             f = re.sub(r'.*\)', r'', f)
-            logging.debug('... factor: "{}"'.format(x))
+            logger.debug('... factor: "{}"'.format(x))
         else:
             x = '1.0'
         facs.append(float(x))
@@ -143,7 +146,7 @@ def _parse_form(forms):
             x = re.sub(r'([0-9]*).*', r'\1', f)
             f = re.sub(r'([0-9]*)', r'', f)
         x = int(float(x))
-        logging.debug('... length: "{}"'.format(x))
+        logger.debug('... length: "{}"'.format(x))
         lens.append(x)
         #
         # Precision
@@ -152,7 +155,7 @@ def _parse_form(forms):
         f = re.sub(r'^([0-9]*)', r'', f)
         if x != '':
             x = int(float(x))
-            logging.debug('... precis: "{}"'.format(x))
+            logger.debug('... precis: "{}"'.format(x))
         else:
             x = None
         prec.append(x)
@@ -170,7 +173,7 @@ def _parse_form(forms):
         # lf   float      8     Festkommazahl (ohne Exponent)
         # e    float      4     Gleitkommazahl (mit Exponent)
         # le   float      8     Gleitkommazahl (mit Exponent)
-        # t    integer    4     Binär:Zeitangabe (ohne Datum):
+        # t    integer    4     Binär: Zeitangabe (ohne Datum):
         #                         vergangene Sekunden
         #                       Text: dd.hh:mm:ss oder hh:mm:ss
         # lt   float      8     Binär: Zeitangabe mit Datum:
@@ -194,7 +197,7 @@ def _parse_form(forms):
                  'f', 'lf', 'e', 'le', 't', 'lt',
                  's'
                  ]:
-            logging.debug('... specif: "{}"'.format(f))
+            logger.debug('... specif: "{}"'.format(f))
             specs.append(f)
             nbyte.append(_BINL[f])
         else:
@@ -211,10 +214,10 @@ def _parse_sequ(dims, sequ, lowb, hghb):
     index position is position of axis in list seq
     e.g. x-axis boundaries are in first column in lowb/highb
          x-index "i" is found in last position, direction is +
-                -> fastest counting, increasing
-                -> along data rows, lowes x left highest x right
+                - fastest counting, increasing
+                - along data rows, lowes x left highest x right
     """
-    logging.debug('sequ: {}'.format(sequ))
+    logger.debug('sequ: {}'.format(sequ))
     sequ = sequ.split(',')
     if len(sequ) != dims:
         print(sequ, len(sequ), dims, len(sequ) - dims)
@@ -247,9 +250,9 @@ def _parse_sequ(dims, sequ, lowb, hghb):
         lowb = [lowb]
     ilen = [x - y + 1 for x, y in zip(hghb, lowb)]
 
-    logging.debug('ipos:   {}'.format(ipos))
-    logging.debug('idir:   {}'.format(idir))
-    logging.debug('ilen:   {}'.format(ilen))
+    logger.debug('ipos:   {}'.format(ipos))
+    logger.debug('idir:   {}'.format(idir))
+    logger.debug('ilen:   {}'.format(ilen))
     return ipos, idir, ilen
 
 
@@ -265,7 +268,7 @@ def _simplify_form(fmt):
 def _to_3d(arr):
     dims = len(np.shape(arr))
     if dims == 1:
-        res = arr[np.newaxis, np.newaxis, :]
+        res = arr[np.newaxis,  :, np.newaxis]
     elif dims == 2:
         res = arr[np.newaxis, :, :]
     elif dims == 3:
@@ -274,7 +277,18 @@ def _to_3d(arr):
         raise ValueError('illegal number of dimensions: %d', dims)
     return res
 
+def _count_digits(arr: np.ndarray) -> int:
+    # this code produces Warnings for 0.0 in arr
+    # np.max(np.ceil(np.log10(np.abs(arr))))
+    #
+    # this code counts actual digits
+    bb = arr.copy()
+    bb[~np.isfinite(bb)] = 0
 
+    lx = len(str(int(np.max(bb))))
+    ln = len(str(int(np.min(bb))))
+    res = max(lx, ln)
+    return res
 #
 #
 #
@@ -293,35 +307,35 @@ class DataFile(object):
        or missing, the raw file contents are discarded after parsing.
      """
     file = None
-    ''' name of file loaded into object '''
+    """ name of file loaded into object """
     text = None
-    ''' text contents the file loaded with the (decompressed)
-      text contents of an eventual external `datfile` appended '''
+    """ text contents the file loaded with the (decompressed)
+      text contents of an eventual external `datfile` appended """
     header = dict()
-    ''' dictionary containing the dmna header entries as strings'''
+    """ dictionary containing the dmna header entries as strings"""
     data_file = None
-    ''' filename if the data block is stored in a separate file '''
+    """ filename if the data block is stored in a separate file """
     binary = False
-    ''' if data block is text of binary data '''
+    """ if data block is text of binary data """
     compressed = False
-    ''' If data block is compressed with gz '''
+    """ If data block is compressed with gz """
     filetype = None
-    ''' `grid` or `timeseries` '''
+    """ `grid` or `timeseries` """
     dims = 0
-    ''' Number of dimensions '''
+    """ Number of dimensions """
     vars = None
-    ''' Number of variables in file  '''
+    """ Number of variables in file  """
     shape = None
-    ''' Shape of data files in `data`  '''
+    """ Shape of data files in `data`  """
     variables = None
-    ''' variable names '''
+    """ variable names """
     data = None
-    ''' dictionary containing the data from the file loaded.
+    """ dictionary containing the data from the file loaded.
       The keys are the variable names.
       The values are of type ``pandas.DataFrame`` with time as index,
       if the file contains timeseries.
       The values are of type ``numpy.array``,
-      if the file contains gridded data. '''
+      if the file contains gridded data. """
     _locl = "C"
     _variable_type = dict()
 
@@ -593,8 +607,7 @@ class DataFile(object):
                             == 0):
                     self._variable_type[var] = "d"
                 else:
-                    digits = np.max(np.ceil(np.log10(np.abs(
-                        self.data[var]))))
+                    digits = _count_digits(self.data[var])
                     if digits > 7 or digits < 0:
                         self._variable_type[var] = "e"
                     else:
@@ -603,11 +616,10 @@ class DataFile(object):
             # determine variable format
             #
             if self._variable_type[var] == "d":
-                digits = np.max(np.ceil(np.log10(np.abs(self.data[var]))))
-                digits = max(digits, 4)
+                digits = max(_count_digits(self.data[var]), 4)
                 fmt = '%%%dhd' % digits
             elif self._variable_type[var] == "f":
-                digits = np.max(np.ceil(np.log10(np.abs(self.data[var]))))
+                digits = _count_digits(self.data[var])
                 if np.all(self.data[var] - np.floor(self.data[var]) == 0):
                     precision = 0
                     digits = max(digits, 5)
@@ -636,14 +648,14 @@ class DataFile(object):
         #
         #  write file
         #
-        logging.info('writing dmna: %s' % filename)
+        logger.info('writing dmna: %s' % filename)
         #
         #  consistency check
         #
         if set(self.variables) != set(self.data.keys()):
             raise ValueError('variable names do match data dict keys')
         valforms = self._attr('form')
-        logging.debug('valforms: ' + str(valforms))
+        logger.debug('valforms: ' + str(valforms))
         if isinstance(valforms, str):
             valforms = [valforms]
         (valnams, _, vallens, _, valspecs, valbyte) = _parse_form(valforms)
@@ -664,24 +676,28 @@ class DataFile(object):
             filename = filename + '.dmna'
         self.header['file'] = os.path.splitext(
             os.path.basename(filename))[0]
-        logging.debug('writing header to file: %s' % filename)
+        logger.debug('writing header to file: %s' % filename)
         data_file, gz = self._get_datfile(filename)
-        logging.debug('writing data to file: %s' % data_file)
+        logger.debug('writing data to file: %s' % data_file)
         mode = self._att1('mode', 'text')
         #
         # open files
         #
-        con1 = open(filename, "w")
+        con1 = open(filename, "w", newline='\r\n')
         if filename != data_file:
             cmpr = self._att1('cmpr', None)
             if mode == 'binary':
-                filemode = "wb"
+                if cmpr is None or cmpr == 0:
+                    con2 = open(data_file, "wb")
+                else:
+                    con2 = gzip.open(data_file, "wb", compresslevel=6)
             else:
-                filemode = "w"
-            if cmpr is None or cmpr == 0:
-                con2 = open(data_file, filemode)
-            else:
-                con2 = gzip.open(data_file, filemode, compresslevel=6)
+                if cmpr is None or cmpr == 0:
+                    con2 = open(data_file, "w",
+                                newline='\r\n')
+                else:
+                    con2 = gzip.open(data_file, "w", compresslevel=6,
+                                     newline='\r\n')
         else:
             con2 = con1
         #
@@ -703,17 +719,17 @@ class DataFile(object):
                     # characters surrounded by quotes
                     value = '  '.join(['"%s"' % x for x in value])
                 lines.append('  '.join((key, value)))
-                logging.debug('header: %s' % lines[-1])
-        con1.writelines([x + '\r\n' for x in lines])
-        con1.writelines(['*' + '\r\n'])
+                logger.debug('header: %s' % lines[-1])
+        con1.writelines([x + '\n' for x in lines])
+        con1.write('*' + '\n')
         #
         # write data body (type specific)
         #
         if self.filetype == 'grid':
-            logging.debug('writing fiel type: grid')
+            logger.debug('writing fiel type: grid')
             values = [self.data[x] for x in self.variables]
         elif self.filetype == 'timeseries':
-            logging.debug('writing file type: timeseries')
+            logger.debug('writing file type: timeseries')
             values = [self.data[x].to_numpy() for x in self.variables]
         else:
             raise ValueError("illegal type: %s" % self.filetype)
@@ -740,15 +756,15 @@ class DataFile(object):
                 np.transpose(values[nv], axes=reverse_ipos))
             out_shape = np.shape(out_values[-1])
         del values
-        logging.debug('out_values shape: %s' % str(out_shape))
+        logger.debug('out_values shape: %s' % str(out_shape))
         #
         #  text mode
         if mode == 'text':
-            logging.debug('writing file mode: text')
+            logger.debug('writing file mode: text')
             #
             # ensure shape has len 3
             if len(out_shape) == 1:
-                out_shape = (1, 1) + out_shape
+                out_shape = (1,) + out_shape +(1,)
             elif len(out_shape) == 2:
                 out_shape = (1,) + out_shape
             elif len(out_shape) == 3:
@@ -764,9 +780,10 @@ class DataFile(object):
                 #
                 # block separator
                 if layer > 0 and out_shape[1] > 1:
-                    con1.writelines(['*' + '\r\n'])
+                    con2.write('*' + '\n')
                 #
                 # write lines for each y grid line (2. dim)
+                lines=[]
                 for nl in range(out_shape[1]):
                     # write group for each x grid line (1. dim)
                     groups = []
@@ -801,10 +818,10 @@ class DataFile(object):
                                 raise ValueError('cannot convert: %s' %
                                                  format(value))
                             groups.append(field)
-                    line = '  ' + ' '.join(groups)
-                    con2.writelines(line + '\r\n')
+                    lines.append('  ' + ' '.join(groups) + '\n')
+                con2.writelines(lines)
         elif mode == 'binary':
-            logging.debug('writing file mode: binary')
+            logger.debug('writing file mode: binary')
             # put all values in big number stream
             numrec = np.size(out_values[0])
             numbers = [None] * (nval * numrec)
@@ -834,7 +851,7 @@ class DataFile(object):
         #
         # write footer
         if mode == 'text':
-            con2.writelines(['***' + '\r\n'])
+            con2.write('***' + '\n')
         if con1 == con2:
             con1.close()
         else:
@@ -851,7 +868,7 @@ class DataFile(object):
         """
         try:
             divider = self.text.index("*")
-            logging.debug('divider: {}'.format(divider))
+            logger.debug('divider: {}'.format(divider))
         except ValueError:
             raise RuntimeError("{} is not in DMNA format".format(self.file))
         #
@@ -863,14 +880,14 @@ class DataFile(object):
                         for x in self.text[0:divider]
                         if not x.strip() == '' and not x.startswith('-')]
         # convert space behind line tag into tab (if not already present)
-        header_lines = [re.sub('\\ +', '\t', x) for x in header_lines]
-        logging.debug([x for x in header_lines])
+        header_lines = [re.sub(' +', '\t', x) for x in header_lines]
+        logger.debug([x for x in header_lines])
         # 1st field is name 2nd and on is content
         header = {}
         for hl in header_lines:
             kv = hl.split('\t', 1)
             if len(kv) < 2:
-                logging.warning('error in header line: "%s"' % hl)
+                logger.warning('error in header line: "%s"' % hl)
             else:
                 header[kv[0]] = kv[1]
         # remove tabs and quotes
@@ -880,7 +897,7 @@ class DataFile(object):
         header['_lines'] = divider
 
         for k, v in header.items():
-            logging.debug('{:6s} {}'.format(k, v))
+            logger.debug('{:6s} {}'.format(k, v))
         return header
 
     # ----------------------------------------------------------------------
@@ -917,7 +934,7 @@ class DataFile(object):
             locl = self._locl
         except AttributeError:
             locl = 'C'
-        logging.debug('looking for key: {}'.format(key))
+        logger.debug('looking for key: {}'.format(key))
         if key in self.header.keys():
             # if key is present: use value
             value = self.header[key]
@@ -930,7 +947,7 @@ class DataFile(object):
         else:
             # if key is not present and no default is set: fail
             raise ValueError('key "{}" not found in header'.format(key))
-        logging.debug('contains value: {}'.format(value))
+        logger.debug('contains value: {}'.format(value))
         if value is None:
             return value
         # split value into space-separated fields
@@ -948,7 +965,7 @@ class DataFile(object):
             try:
                 if v == 'None':
                     v = None
-                    logging.debug('... field {:02d} is None'.format(i))
+                    logger.debug('... field {:02d} is None'.format(i))
                 else:
                     if pd.api.types.is_numeric_dtype(v):
                         v = float(v)
@@ -956,13 +973,13 @@ class DataFile(object):
                         v = _locl_float(v, locl)
                     if v.is_integer():
                         v = int(v)
-                        logging.debug(
+                        logger.debug(
                             '... field {:02d} is int  : {:d}'.format(i, v))
                     else:
-                        logging.debug(
+                        logger.debug(
                             '... field {:02d} is float: {:f}'.format(i, v))
             except ValueError:
-                logging.debug('... field {:02d} is text : {:s}'.format(i, v))
+                logger.debug('... field {:02d} is text : {:s}'.format(i, v))
             res.append(v)
         return res
 
@@ -997,7 +1014,7 @@ class DataFile(object):
             sk = axes['sk']
         elif 'z' in axes.keys():
             dims = 3
-            sk = axes['Z']
+            sk = axes['z']
         else:
             sk = None
         #
@@ -1102,10 +1119,10 @@ class DataFile(object):
             filename = self.file
         # ascii or binary ?
         mode = self._att1('mode', 'text')
-        logging.debug('mode: {}'.format(mode))
+        logger.debug('mode: {}'.format(mode))
         # compression strength ?
         cmpr = self._att1('cmpr', '0')
-        logging.debug('cmpr: {}'.format(cmpr))
+        logger.debug('cmpr: {}'.format(cmpr))
         #
         # name of separate datafile (if any)
         if mode == 'text' and cmpr > 0:
@@ -1119,7 +1136,7 @@ class DataFile(object):
         else:
             raise ('illegal data file mode/compression: %s/%s' %
                    (mode, str(cmpr)))
-        logging.debug('datfile: {}'.format(data_file))
+        logger.debug('datfile: {}'.format(data_file))
         if cmpr > 0:
             gz = True
         else:
@@ -1156,23 +1173,23 @@ class DataFile(object):
              ) = _parse_form(form)
             nval = len(valspec)
 
-            logging.debug('nval:   {}'.format(nval))
-            logging.debug('valnams : {}'.format(valnams))
-            logging.debug('valfacs : {}'.format(valfacs))
-            logging.debug('vallens : {}'.format(vallens))
-            logging.debug('valprec : {}'.format(valprec))
-            logging.debug('valspecc: {}'.format(valspec))
+            logger.debug('nval:   {}'.format(nval))
+            logger.debug('valnams : {}'.format(valnams))
+            logger.debug('valfacs : {}'.format(valfacs))
+            logger.debug('vallens : {}'.format(vallens))
+            logger.debug('valprec : {}'.format(valprec))
+            logger.debug('valspec : {}'.format(valspec))
         else:
             nval = 1
             valnams = valspec = [""]
         #
         # ascii or binary ?
         mode = self._att1('mode', 'text')
-        logging.debug('mode: {}'.format(mode))
+        logger.debug('mode: {}'.format(mode))
         #
         # number format ?
         locl = self._att1('locl', 'C')
-        logging.debug('locl: {}'.format(mode))
+        logger.debug('locl: {}'.format(mode))
         #
         # select file opening function according to compression
         if self.compressed:
@@ -1211,7 +1228,7 @@ class DataFile(object):
                     # parse lines
                     if '*' in line:
                         # stars denote block boundaries
-                        logging.debug(
+                        logger.debug(
                             'stopped reading at line {} ("{}")'.format(nl, line))
                         break
                     elif line.strip() == '':
@@ -1242,7 +1259,7 @@ class DataFile(object):
                                 nxt = np.datetime64(
                                     field.replace('.', ' '))
                             elif spec in ['s']:
-                                nxt = [int(x) for x in field.strip('" ')]
+                                nxt = field.strip('" ')
                             else:
                                 raise RuntimeError(
                                     'internal: illegal format ' +
@@ -1263,9 +1280,9 @@ class DataFile(object):
         else:
             raise IOError('unsupported mode: {}'.format(mode))
 
-        logging.debug('numrec : {}'.format(numrec))
-        logging.debug('#values: {}'.format(numrec * nval))
-        logging.debug('#read  : {}'.format(len(numbers)))
+        logger.debug('numrec : {}'.format(numrec))
+        logger.debug('#values: {}'.format(numrec * nval))
+        logger.debug('#read  : {}'.format(len(numbers)))
 
         # split variables to individual fields:
         # 123123123123 -> [1111],[2222],[3333]
@@ -1321,7 +1338,7 @@ class DataFile(object):
                 'te' in data.columns):
             return data
         # do correction
-        logging.debug('filetype axes=ti: adding time column')
+        logger.debug('filetype axes=ti: adding time column')
         dt = _parsedifftime(header['dt'])  # "01:00:00"
         if 't1' in header.keys():
             t1 = _parsedifftime(header['t1'])  # "00:00:00"
@@ -1344,7 +1361,7 @@ class DataFile(object):
 
         rdat = pd.to_datetime(rd, utc=True)
         te = pd.date_range(start=rdat + t1, end=rdat + t2 - dt, freq=dt)
-        logging.debug('... %s -- %s' % (te[0].strftime("%F %T"),
+        logger.debug('... %s -- %s' % (te[0].strftime("%F %T"),
                                         te[-1].strftime("%F %T")))
         res = pd.DataFrame({'te': te})
         points = header['mntn'].split()
@@ -1370,6 +1387,7 @@ class DataFile(object):
         :param header_only: (optional) If ``True`` skip loading the data. \
           Useful for fast scanning of file headers.
         """
+        logger.info('loading file: %s' % file)
         for en in _ENCODINGS:
             try:
                 with codecs.open(self.file, 'r', encoding=en) as f:
@@ -1380,7 +1398,7 @@ class DataFile(object):
                         self.text.append(str(x).rstrip('\n').rstrip('\r'))
                         if header_only and self.text[-1].strip() == '*':
                             break
-                logging.debug('file encoding: %s' % en)
+                logger.debug('file encoding: %s' % en)
                 break
             except UnicodeDecodeError:
                 continue
@@ -1418,10 +1436,10 @@ class DataFile(object):
         """
         axes = self._att1('axes', None)
         dims = self._att1('dims', -1)
-        print('axes : %s' % format(axes))
+        logger.debug('axes : %s' % format(axes))
         if axes == 'ti' or dims == 1:
             return self.data['te'].values
-        elif axes in ['xy', 'xyz', 'xyzs']:
+        elif axes in ['xy', 'xyz', 'xyzs', None]:
             return self._get_axes(ax)
 
     # ----------------------------------------------------------------------
